// Tree-ring records DataEditor
// 2013.8.21
// by MATHRAX and FUJITSU
//
// Creative Commons License (CC-BY 3.0)
// http://mathrax.com/lab/?p=1326
//
// Processing 2.0.1


//GUI
import controlP5.*;
ControlP5 cp5;

//SOUND
import ddf.minim.*;
Minim minim;
AudioSample orgel[];

//IMAGE
PImage imageFile;

//Global Variable
int speed = 1;              //rotate speed
int color_difference = 10;  //color difference
color lastColor[]; 
float alphaList[];


//MUSIC DATA
int startTime;
int[] millisec=new int[0];
int[] soundNumber=new int[0];

//FIRST PARAMETER
int firstData[]=new int[3];

//SAVE parameter
String[] loadlines;

//IMAGE ROTATE
int r;
int rotate_START;

//ROTATE DELAY
int prescale;

//FIRST BTN CANCEL
int firstExportBtnCancel=0;




// SETUP ---------------------------------------------------
void setup() {
  //FULL SCREEN BACKGROUND COLOR
  //frame.setBackground(new java.awt.Color(0, 0, 0));
  
  frameRate(60);
  size(640, 400, P3D);

  //LOAD PARAMETER
  loadlines = loadStrings("data/points.txt");
  if (loadlines!=null) {
    String[] pieces = split(loadlines[0], '\t');
    if (pieces.length == 3) {
      firstData[0]           = int(pieces[0]);
      firstData[1]           = int(pieces[1]);
      firstData[2]           = int(pieces[2]);
    }
  }
  else {
    firstData[0] =10;
    firstData[1] =20;
    firstData[2] =0;
  }

  //LOAD SOUNDS
  minim = new Minim(this);
  orgel = new AudioSample[16];

  //SAMPLING FROM PIC microchip.
  orgel[0]=minim.loadSample( "s1.wav", 512);
  orgel[1]=minim.loadSample( "s2.wav", 512);
  orgel[2]=minim.loadSample( "s3.wav", 512);
  orgel[3]=minim.loadSample( "s4.wav", 512);
  orgel[4]=minim.loadSample( "s5.wav", 512);
  orgel[5]=minim.loadSample( "s6.wav", 512);
  orgel[6]=minim.loadSample( "s7.wav", 512);
  orgel[7]=minim.loadSample( "s8.wav", 512);
  orgel[8]=minim.loadSample( "s9.wav", 512);
  orgel[9]=minim.loadSample( "s10.wav", 512);
  orgel[10]=minim.loadSample( "s11.wav", 512);
  orgel[11]=minim.loadSample( "s12.wav", 512);
  orgel[12]=minim.loadSample( "s13.wav", 512);
  orgel[13]=minim.loadSample( "s14.wav", 512);
  orgel[14]=minim.loadSample( "s15.wav", 512);
  orgel[15]=minim.loadSample( "s16.wav", 512);


  //GLOBAL VARIABLE PRESET
  lastColor = new color[16];
  alphaList = new float[16];
  for (int i=0; i<16; i++) {
    lastColor[i]=0;
    alphaList[i]=0;
  }

  //IMAGE PRESET
  imageMode(CENTER);
  ellipseMode(CENTER);
  imageFile = loadImage("nenrin_1.gif");
  //imageFile = loadImage("nenrin_2.gif");
  image(imageFile, width/2, height/2);


  //GUI setting
  cp5 = new ControlP5(this);
  //SLIDER (SPEED)
  cp5.addSlider("speed")
    .setPosition(30, 350)
      .setSize(200, 20)
        .setRange(1, 20)
          .setValue(firstData[0])
            .setColorValue(color(255))
              .setColorActive(color(200, 150, 60))
                .setColorForeground(color(155, 100, 10))
                  .setColorBackground(color(20, 10, 10))
                    ;
  cp5.getController("speed").getValueLabel().align(ControlP5.LEFT, ControlP5.BOTTOM_OUTSIDE).setPaddingX(0);
  cp5.getController("speed").getCaptionLabel().align(ControlP5.RIGHT, ControlP5.BOTTOM_OUTSIDE).setPaddingX(0);


  //SLIDER (COLOR DIFFERENCE)
  cp5.addSlider("color_difference")
    .setPosition(280, 350)
      .setSize(200, 20)
        .setRange(0, 255)
          .setValue(firstData[1])
            ;
  cp5.getController("color_difference").getValueLabel().align(ControlP5.LEFT, ControlP5.BOTTOM_OUTSIDE).setPaddingX(0);
  cp5.getController("color_difference").getCaptionLabel().align(ControlP5.RIGHT, ControlP5.BOTTOM_OUTSIDE).setPaddingX(0);


  //START BUTTON
  cp5.addToggle("START")
    .setValue(0)
      .setPosition(520, 350)
        .setSize(50, 20)
          .setLabel("START");
  ;
  cp5.getController("START").getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER).setPaddingX(0);


  //EXPORT BUTTON
  cp5.addButton("Export")
    .setValue(0)
      .setPosition(571, 350)
        .setSize(50, 20)
          ;
  cp5.getController("Export").getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER).setPaddingX(0);


  r=firstData[2];
  startTime=millis();
}
// end of SETUP ---------------------------------------------------




// DRAW -----------------------------------------------------------
void draw() {
  background(0);

  //IMAGE
  pushMatrix();
  translate(width/2, 320);
  rotate(radians(r));
  image(imageFile, 0, 0);
  popMatrix();

  //ROTATE SPEED
  prescale++;
  if (prescale%(21-speed)==0) {
    if (rotate_START==1) r++;
    if (r>=360)r=0;
  }


  stroke(255, 100);
  line(width/2, 0, width/2, 320);

  if (rotate_START==1) {
    for (int i=0; i<=15; i++) {

      noStroke();
      color c = get(width/2, 320-(i*20+10));

      fill(0, 255, 255, 150);
      ellipse(width/2, 320-(i*20+10), 5, 5);

      if (abs(lastColor[i]&0xFF - c&0xFF)>color_difference) {
        alphaList[i]=255;

        //PLAY SOUND
        orgel[i].trigger();

        //DATA SET
        soundNumber = append(soundNumber, i);
        millisec = append(millisec, millis()-startTime);
      }
      else {
        alphaList[i]*=0.9;
        fill(c, alphaList[i]);
      }

      fill(255, alphaList[i]);
      ellipse(width/2, 320-(i*20+10), 7, 7);

      lastColor[i] = c;
    }
  }

  //GUI-FOOTER
  pushMatrix();
  translate(0, 320);
  noStroke();
  fill(0, 100);
  rect(0, 0, width, 130);
  popMatrix();
}
// end of DRAW ----------------------------------------------------


// ControlP5 STARTorSTOP BUTTON -----------------------------------
public void START(int theValue) {

  rotate_START=theValue;

  //START
  if (theValue==1) {
    cp5.getController("START").setLabel("START");
    resetData();
  }
  //STOP
  else {
    cp5.getController("START").setLabel("STOP");
  }


  startTime=millis();
}


// ControlP5 EXPORT BUTTON ----------------------------------------
public void Export(int theValue) {

  //ROTATE_STOP
  rotate_START=0;
  cp5.getController("START").setLabel("STOP");
  cp5.getController("START").setValue(0);

  if (firstExportBtnCancel!=0) {

    selectFolder("Select a save folder:", "folderSelected");
  }
  firstExportBtnCancel=1;
}


//DATA RESET
void resetData() {
  int[] array = new int[0];
  soundNumber=array;
  millisec=array;
}


//DATA SAVE FOLDER SELECT
void folderSelected(File selection) {
  if (selection!=null) {
    String[] lines = new String[soundNumber.length];
    for (int i = 0; i < soundNumber.length; i++) {
      lines[i] = millisec[i] + ",\t" + soundNumber[i]+",";
    }

    //SAVE DATA
    saveStrings(selection+"/"+"records"+year()+nf(month(), 2)+nf(day(), 2)+"_"+nf(hour(), 2)+nf(minute(), 2)+nf(second(), 2)+".txt", lines);

    saveParameterSet(speed, color_difference, r);
    println("EXPORT");
  }
}


//SAVE PARAMETER
void saveParameterSet(int a, int b, int c) {
  String[] parameterlines = new String[1];

  parameterlines[0] = a + "\t" + b + "\t" + c ;

  saveStrings("data/points.txt", parameterlines);
}

